% gmm.m
% returns gmm estimates and standard errors
% b=gmm(b0,Ydata,which_moments,efficient,V);
% if efficient==1, uses efficient weighting matrix
% if efficient==0, uses identity weighting matrix
% if V==1, include variance-covariance matrix

function [G,varargout]=gmm(b0,Ydata,which_moments,efficient,V);

global alpha beta sigma delta BYbar gamma psi Gbar0 phi0 rhog0 rhoz0 use_uhlig impulse_response;

addpath burnside;

T=length(Ydata(:,1));


lags=6; %lags for Newey West

b_eye=fminsearch('gmmsum',b0,[],[],Ydata,which_moments);

u=gmmresid(b_eye,Ydata,which_moments);

fit=gmmsum(b_eye,[],Ydata,which_moments);

%calculate spectral matrix

s0=s0calc(u,2,lags);


if efficient==1;

	b_gmm=fminsearch('gmmsum',b_eye,[],s0^(-1),Ydata,which_moments);

	dg=dfeval('mgmmerr',mgmmerr(b_gmm,Ydata,which_moments),b_gmm,0,Ydata,which_moments);

	Var=(dg'*s0^(-1)*dg)^(-1)/T; 
	se=sqrt(diag(Var));
	J=gmmsum(b_gmm,s0^(-1),Ydata,which_moments);
   pvalue=1-chi2cdf(T*J,size(u,2)-length(b_gmm));
   
elseif efficient==0;
   b_gmm=b_eye;
	dg=dfeval('mgmmerr',mgmmerr(b_gmm,Ydata,which_moments),b_gmm,0,Ydata,which_moments);

   Var=(dg'*dg)^(-1)*(dg'*s0*dg)/T*(dg'*dg)^(-1);
   se=sqrt(diag(Var));
   J=NaN;
   pvalue=NaN;
   
end;



% standard error for relative permanent component
% F(b) = ((1-rhog)/(1+rhog)+(1-rhog^2)/(1+rhoz)*2/alpha*sigmaz^2/sigmag^2)^(-1);

sz=b_gmm(1);
sg=b_gmm(2);
rz=rhoz0;
rg=rhog0;

if length(b_gmm)==2;
   F = ((1-rg)/(1+rg)+ (1-rg)^2/(1+rz)*2/alpha^2*sz^2/sg^2)^(-1);
   Fprime = [-F^2*4*sz*(1-rg)^2/((1+rz)*alpha^2*sg^2),...
         4*sz^2*(1-rg)^2*F^2/(alpha^2*(1+rz)*sg^3)];
   var_F=Fprime*Var(1:2,1:2)*Fprime';
   se_F=sqrt(var_F);
   
elseif length(b_gmm)==3;
   rg=b_gmm(3);
   F = ((1-rg)/(1+rg)+ (1-rg)^2/(1+rz)*2/alpha^2*sz^2/sg^2)^(-1);
   Fprime = [-F^2*4*sz*(1-rg)^2/((1+rz)*alpha^2*sg^2),...
         4*sz^2*(1-rg)^2*F^2/(alpha^2*(1+rz)*sg^3),...
      F^2*(1/(1+rg)+(1-rg)/(1+rg)^2+4*sz^2*(1-rg)/((1+rz)*alpha^2*sg^2))];
   
   var_F=Fprime*Var*Fprime';
   se_F=sqrt(var_F);
   
   
elseif length(b_gmm)>=4 ;
   rz=b_gmm(3);
   rg=b_gmm(4);
   
  F = ((1-rg)/(1+rg)+ (1-rg)^2/(1+rz)*2/alpha^2*sz^2/sg^2)^(-1);
   Fprime = [-F^2*4*sz*(1-rg)^2/((1+rz)*alpha^2*sg^2),...
         4*sz^2*(1-rg)^2*F^2/(alpha^2*(1+rz)*sg^3),...
      2*sz^2*(1-rg)^2*F^2/((1+rz)^2*alpha^2*sg^2),...   
      F^2*(1/(1+rg)+(1-rg)/(1+rg)^2+4*sz^2*(1-rg)/((1+rz)*alpha^2*sg^2))];



var_F=Fprime*Var(1:4,1:4)*Fprime';
   se_F=sqrt(var_F);
  
end;

   G=[b_gmm(:),se(:),b_eye(:);
   F,se_F,0;
   J,pvalue,fit*10e+7];



if nargout==2;
   varargout{1}=Var;
end;
