% gmmresid(b,Y)
% calculates difference (residuals) between theoretical moments and data
% Y should be a Tx5 vector of filtered output,consumption, investment and net exports,
% and unfiltered growth
% which_moments = (
% 1  stdev(y)
% 2  stdev(delta y)
% 3  stdev(inv)/stdev(y)
% 4  stdev(c)/stdev(y)
% 5  stdev(nx)/stdev(y)
% 6  rho(y,y(-1))
% 7  rho(delta y, delta y(-1)) 
% 8  rho(nx,y)
% 9  rho(c,y)
% 10 rho(inv,y) 
% 11 mu_g)
% if which_moments(n)==1, then that moment is used.
% otherwise, set which_moments==0;


function u=gmmresid(b,Y,which_moments);

%global Ydata;

%demean data (except growth):


Y(:,1:4)=Y(:,1:4)-ones(length(Y),1)*mean(Y(2:end,1:4));


y=Y(:,1);
c=Y(:,2);
inv=Y(:,3);
nx=Y(:,4);
dy=Y(:,5);

m=key_moments(b);


u1=(m(1)/100).^2-y(2:end).^2; %stdev(y)
if length(b)>5;
   u2=(m(2)/100).^2-(dy(2:end)-log(b(5))).^2; %stdev(delta y)
else;
   u2=(m(2)/100).^2-(dy(2:end)-mean(dy(2:end))).^2; %stdev(delta y)
end;
%u3=m(3)^2*y(2:end).^2-inv(2:end).^2; %stdev(inv)/stdev(y)
%u3=m(3)^2-inv(2:end).^2/var(y(2:end),1); %stdev(inv)/stdev(y)
u3=m(3)^2*(m(1)/100).^2-inv(2:end).^2; %stdev(inv)
%u4=m(4)^2*y(2:end).^2-c(2:end).^2; %stdev(c)/stdev(y)
%u4=m(4)^2-c(2:end).^2/var(y(2:end),1); %stdev(c)/stdev(y)
u4=m(4)^2*(m(1)/100).^2-c(2:end).^2; %stdev(c)
%u5=m(5)^2*y(2:end).^2-nx(2:end).^2; %stdev nx /stdev(y)
u5=m(5)^2*(m(1)/100).^2-nx(2:end).^2; %stdev nx 
%u6=m(6)*y(1:end-1).^2-y(2:end).*y(1:end-1); %rho(y,y(-1))
u6=m(6)*(m(1)/100)^2-y(2:end).*y(1:end-1); %cov(y,y(-1))
%u6=y(2:end)-m(6)*y(1:end-1); %rho(y,y(-1))
if length(b)>5;
   u7=m(7)*(m(2)/100)^2-(dy(1:end-1)-log(b(5))).*(dy(2:end)-log(b(5))); %cov(delta y, delta y(-1))
else;
   u7=m(7)*(m(2)/100)^2-(dy(1:end-1)-mean(dy(1:end-1))).*(dy(2:end)-mean(dy(2:end))); %cov(delta y, delta y(-1))
end;
%u7=dy(2:end)-m(7)*dy(1:end-1)-(1-m(7))*mean(dy(2:end)); %rho(y,y(-1))
u8=m(8)*(m(1)/100).^2*m(5)-y(2:end).*nx(2:end); %cov(nx,y)
%u8=m(8)-y(2:end).*nx(2:end)/(std(y(2:end),1)*std(nx(2:end),1)); %rho(nx,y)
%u8=m(8)*(std(y(2:end),1)*std(nx(2:end),1))-y(2:end).*nx(2:end); %rho(nx,y)
u9=m(9)*(m(1)/100).^2*m(4)-y(2:end).*c(2:end); %cov(c,y)
%u9=m(9)-y(2:end).*c(2:end)/(std(y(2:end),1)*std(c(2:end),1)); %rho(c,y)
%u9=m(9)*(std(y(2:end),1)*std(c(2:end),1))-y(2:end).*c(2:end); %rho(c,y)
u10=m(10)*(m(1)/100).^2*m(3)-y(2:end).*inv(2:end); %cov(inv,y)
%u10=m(10)-y(2:end).*inv(2:end)/(std(y(2:end),1)*std(inv(2:end),1)); %rho(inv,y)
%u10=m(10)*(std(y(2:end),1)*std(inv(2:end),1))-y(2:end).*inv(2:end); %rho(inv,y)

if length(b)>5;
   u11=log(b(5))-dy(2:end); %mu_g
else;
   u11=NaN*ones(length(y)-1,1);
end;

u_full = [u1(:),u2(:),u3(:),u4(:),u5(:),u6(:),u7(:),u8(:),u9(:),u10(:),u11(:)];

u=u_full(:,find(which_moments(:)'==1));






