% gmmsum(b,W,Y,which_moments)
% b is the vector of parameters
% W is the weighting matrix.  Use [] if identity matrix.
% Y should be a Tx5 vector of filtered output,consumption, investment and net exports,
% and unfiltered growth
% calculates weighted sum of squared residuals from gmmresid
% If b is inadmissable (i.e., parameters negative, ar(1) coefficients>=1, or mu_g<0.99), then J set to 1e+10
% which_moments = (stdev(y),stdev(delta y),stdev(inv)/stdev(y), stdev(c)/stdev(y),...
% stdev(nx)/stdev(y),rho(y,y(-1)),rho(delta y, delta y(-1)),rho(nx,y), rho(c,y), rho(inv,y), mu_g)
% if which_moments(n)==1, then that moment is used.
% otherwise, set which_moments==0;


function J=gmmsum(b,W,Y,which_moments);


if length(b)==2;
   admissable = min(b)>=0;
elseif length(b)==3;
	admissable = min(b(1:2))>=0 & b(3)>=0 & b(3)<1-(1e-6);   
elseif length(b)==4 | length(b)==5;
   admissable = min(b(1:2))>=0 & min(b(3:4))>=0 & max(b(3:4))<1-(1e-6);
elseif length(b)==6;
   admissable = min(b(1:2))>=0 & min(b(3:4))>=0 & max(b(3:4))<1-(1e-6) & b(5)>=.99;
end;
   


if isempty(W)==1;
   W=eye(sum(which_moments));
end;


u=gmmresid(b,Y,which_moments);

mu=mean(u);


if admissable==1;
   J=mu(:)'*W*mu(:);
else;
   J=1e+10;
end;


